/*
 * Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package ingest

import (
	"encoding/json"
	"fmt"
	"gitlab.nrao.edu/ssa/gocapo/capo/config"
	"io"
	"io/ioutil"
	"net/http"
	"net/url"
	"path/filepath"
)

type calibrationInfo struct {
	Locator string `json:"locator"`
}

type CapoInput struct {
	CapoPath    string
	CapoProfile string
}

//checkError
/**
 * Check if an error happened and panic on it if so
 *
 * @param err An error object to report
 **/
func checkError(err error) {
	if err != nil {
		panic(err)
	}
}

//getCapoConfig
/**
 * Create a CapoConfig instance from a CAPO property file
 *
 * @param input A CapoInput instance encapsulating the requested profile and path
 * @return a CapoConfig instance
 **/
func getCapoConfig(input CapoInput) config.CapoConfig {
	properties, err := config.InitConfig(input.CapoProfile, input.CapoPath)
	checkError(err)

	return properties
}

//GetCalibrationLocator
/**
 * Get the locator for a VLASS image set
 *
 * @param img_set_path A string with the path to the image set
 * @param capo_profile A string with the path to the CAPO property file used to
 * get the calibration lookup URL
 * @return a string with the locator value
 **/
func GetCalibrationLocator(imgSetPath string, input CapoInput) string {
	settings := getCapoConfig(input).SettingsForPrefix("edu.nrao.archive.workflow.config.IngestionWorkflowSettings")
	locatorEndpoint := settings.GetString("calibrationLookupUrl")

	imgSet := filepath.Base(imgSetPath)

	// Make the REST call
	lookupUrl := locatorEndpoint + url.QueryEscape(imgSet)
	resp, err := http.Get(lookupUrl)
	checkError(err)

	defer func(Body io.ReadCloser) {
		err := Body.Close()
		checkError(err)
	}(resp.Body)

	if resp.StatusCode != http.StatusOK {
		panic(fmt.Sprintf("Error making GET request to URL %s, got status code %d", lookupUrl, resp.StatusCode))
	}

	respBytes, err := ioutil.ReadAll(resp.Body)
	checkError(err)

	// Get the json of the response body
	var calibration calibrationInfo
	err = json.Unmarshal(respBytes, &calibration)
	checkError(err)

	return calibration.Locator
}
