#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
This is a workflow CLI. The purpose here is two-fold:

1. Simplify submitting workflow requests manually
2. Enable debugging of workflows by directly evaluating them
"""

import argparse as ap
import json
from pathlib import Path

from workspaces.workflow.services.workflow_service import WorkflowService


def submit_request(ns: ap.Namespace):
    """
    Submit a request to the REST API for execution.
    :param namespace:
    :return:
    """
    # 1. Look up the REST API location in Capo
    # 2. Encode and submit this request
    # 3. Start listening for workflow events on this request
    # 4. Quit when the final message comes back
    print(ns)
    raise NotImplementedError


def debug_workflow(ns: ap.Namespace):
    """
    Pretend to be a workflow service and evaluate a request ourselves.
    :param namespace:
    :return:
    """
    service = WorkflowService()

    # handle the argument
    arg = None
    if ns.inline_argument:
        arg = json.loads(ns.inline_argument)
    elif ns.argument_file:
        arg = json.load(Path(ns.argument_file))

    # create request
    request = service.info.create_workflow_request(ns.name, arg)

    # ask the service to execute this request
    service.execute(request, ns.file)

    # should hang out printing messages about this
    # request, until we get the "complete" message


def main():
    parser = ap.ArgumentParser("workflow")
    subparsers = parser.add_subparsers()

    # subcommand 1: submit a workflow request
    submit = subparsers.add_parser("submit", help="Submit request to workflow service")
    submit.add_argument("name", nargs=1, help="Name of the workflow to request executing")
    submit.add_argument("file", nargs="*", help="Files to add to the request")
    group = submit.add_mutually_exclusive_group()
    group.add_argument(
        "-i",
        "--inline-argument",
        nargs=1,
        type=str,
        help="Argument to the workflow, as literal JSON string",
    )
    group.add_argument(
        "-f",
        "--argument-file",
        nargs=1,
        type=Path,
        help="Argument to the workflow, as file containing JSON",
    )
    submit.set_defaults(func=submit_request)

    # subcommand 2: debug a workflow request
    debug = subparsers.add_parser("debug", help="Evaluate a workflow service request locally (developer command)")
    debug.add_argument("name", nargs=1, help="Name of the workflow to request executing")
    debug.add_argument("file", nargs="*", help="Files to add to the request")
    group = debug.add_mutually_exclusive_group()
    group.add_argument(
        "-i",
        "--inline-argument",
        nargs=1,
        type=str,
        help="Argument to the workflow, as literal JSON string",
    )
    group.add_argument(
        "-f",
        "--argument-file",
        nargs=1,
        type=Path,
        help="Argument to the workflow, as file containing JSON",
    )
    debug.set_defaults(func=debug_workflow)

    args = parser.parse_args()
    args.func(args)
